#!/usr/bin/env python-for-pytorch

import torch
import math, sys, re

from torch import nn
from torch.nn import functional as fn

from torch import Tensor
from torch.autograd import Variable
from torch.nn.parameter import Parameter
from torch.nn import Module

######################################################################

class Link:
    def __init__(self, from_node, from_nb, to_node, to_nb):
        self.from_node = from_node
        self.from_nb = from_nb
        self.to_node = to_node
        self.to_nb = to_nb

class Node:
    def __init__(self, id, label):
        self.id = id
        self.label = label
        self.max_in = -1
        self.max_out = -1

def slot(node_list, n, k, for_input):
    if for_input:
        if node_list[n].max_out > 0:
            return str(node_list[n].id) + ':input' + str(k)
        else:
            return str(node_list[n].id)
    else:
        if node_list[n].max_in > 0:
            return str(node_list[n].id) + ':output' + str(k)
        else:
            return str(node_list[n].id)

def slot_string(k, for_input):
    result = ''

    if for_input:
        label = 'input'
    else:
        label = 'output'

    if k > 0:
        if not for_input: result = ' |' + result
        result +=  ' { <' + label + '0> 0'
        for j in range(1, k+1):
            result += " | " + '<' + label + str(j) + '> ' + str(j)
        result += " } "
        if for_input: result = result + '| '

    return result

######################################################################

def add_link(node_list, link_list, u, nu, v, nv):
    link = Link(u, nu, v, nv)
    link_list.append(link)
    node_list[u].max_in  = max(node_list[u].max_in,  nu)
    node_list[v].max_out = max(node_list[u].max_out, nv)

######################################################################

def build_ag_graph_lists(u, node_labels, out, node_list, link_list):

    if not u in node_list:
        node = Node(len(node_list) + 1,
                    (u in node_labels and node_labels[u]) or \
                    re.search('<class \'(.*\.|)([a-zA-Z0-9_]*)\'>', str(type(u))).group(2))
        node_list[u] = node

        if isinstance(u, torch.autograd.Variable):
            build_ag_graph_lists(u.grad_fn, node_labels, out, node_list, link_list)
            add_link(node_list, link_list, u, 0, u.grad_fn, 0)
        else:
            if hasattr(u, 'next_functions'):
                i = 0
                for v, j in u.next_functions:
                    build_ag_graph_lists(v, node_labels, out, node_list, link_list)
                    add_link(node_list, link_list, u, i, v, j)
                    i += 1

######################################################################

def print_dot(node_list, link_list, out):
    out.write('digraph{\n')

    for n in node_list:
        node = node_list[n]

        out.write(
            '  ' + \
            str(node.id) + ' [shape=record,label="{ ' + \
            slot_string(node.max_out, for_input = True) + \
            node.label + \
            slot_string(node.max_in, for_input = False) + \
            ' }"]\n'
        )

    for n in link_list:
        out.write('  ' + \
                  slot(node_list, n.from_node, n.from_nb, for_input = False) + \
                  ' -> ' + \
                  slot(node_list, n.to_node, n.to_nb, for_input = True) + \
                  '\n')

    out.write('}\n')

######################################################################

def save_dot(x, node_labels = {}, out = sys.stdout):
    node_list = {}
    link_list = []
    build_ag_graph_lists(x, node_labels, out, node_list, link_list)
    print_dot(node_list, link_list, out)

######################################################################

# x = Variable(torch.rand(5))
# y = torch.topk(x, 3)
# l = torch.sqrt(torch.norm(y[0]) + torch.norm(5.0 * y[1].float()))

# save_dot(l, { l: 'variable l' }, open('/tmp/test.dot', 'w'))
